﻿namespace U2UConsult.Composite.Communication.Sample.RegularEvents
{
    using System;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.ComponentModel.Composition.Hosting;
    using System.Reflection;
    using System.Threading;
    using System.Windows.Input;
    using System.Windows.Threading;

    /// <summary>
    /// Regular Events MVVM ViewModel.
    /// </summary>
    public class ViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Unreference subscriber command.
        /// </summary>
        private ICommand destroy;

        /// <summary>
        /// Publish command.
        /// </summary>
        private ICommand publish;

        /// <summary>
        /// MEF compose command.
        /// </summary>
        private ICommand compose;

        /// <summary>
        /// Memory cleanup timer.
        /// </summary>
        private DispatcherTimer timer;

        /// <summary>
        /// Subscriber.
        /// </summary>
        private Subscriber subscriber;

        /// <summary>
        /// Initializes a new instance of the ViewModel class.
        /// </summary>
        public ViewModel()
        {
            this.compose = new RelayCommand(this.Composed);
            this.publish = new RelayCommand(this.Published);
            this.destroy = new RelayCommand(this.Destroyed);

            this.timer = new DispatcherTimer(TimeSpan.FromSeconds(2), DispatcherPriority.Background, this.Tick, Dispatcher.CurrentDispatcher);

            this.compose.Execute(null);
        }

        /// <summary>
        /// Change notification.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets a publisher.
        /// </summary>
        [Import]
        public IPublisher Publisher { get; set; }

        /// <summary>
        /// Gets or sets a subscriber.
        /// </summary>
        [Import]
        public Subscriber Subscriber
        {
            get 
            { 
                return this.subscriber; 
            }

            set
            {
                this.subscriber = value;
                this.PropertyChanged.Raise(this, o => o.Subscriber);
            }
        }

        /// <summary>
        /// Gets the publish command.
        /// </summary>
        public ICommand Publish
        {
            get { return this.publish; }
        }

        /// <summary>
        /// Gets the compose command.
        /// </summary>
        public ICommand Compose
        {
            get { return this.compose; }
        }

        /// <summary>
        /// Gets the destroy command
        /// </summary>
        public ICommand Destroy
        {
            get { return this.destroy; }
        }

        /// <summary>
        /// Gets the number of subscriber instances in memory.
        /// </summary>
        public int SubscriberInstances
        {
            get
            {
                return Subscriber.InstancesCount;
            }
        }

        /// <summary>
        /// Executes the publish command: lets the publisher publish.
        /// </summary>
        /// <param name="o">Ignored parameter.</param>
        private void Published(object o)
        {
            this.Publisher.Publish();
        }

        /// <summary>
        /// Executes the compose command: composes the MEF object graph.
        /// </summary>
        /// <param name="o">Ignored parameter.</param>
        private void Composed(object o)
        {
            AssemblyCatalog catalog = new AssemblyCatalog(Assembly.GetExecutingAssembly());
            CompositionContainer container = new CompositionContainer(catalog);
            container.ComposeParts(this);
        }

        /// <summary>
        /// Executes the destroy command: removes the reference to the subscriber.
        /// </summary>
        /// <param name="o">Ignored parameter.</param>
        private void Destroyed(object o)
        {
            if (this.subscriber != null)
            {
                this.subscriber.Message = "I'll be back ...";

                // Add a delay, so the message can be displayed properly.
                System.Windows.Forms.Application.DoEvents();
                Thread.Sleep(2500);

                this.Subscriber = null;
            }
        }

        /// <summary>
        /// Triggers garbages collection and updates subscriber count.
        /// </summary>
        /// <param name="sender">Sender of the event: timer.</param>
        /// <param name="e">Event arguments.</param>
        private void Tick(object sender, EventArgs e)
        {
            GC.Collect();
            GC.Collect();

            this.PropertyChanged.Raise(this, v => v.SubscriberInstances);
        }
    }
}
